import json
import logging


class Encoder(json.JSONEncoder):
    def default(self, o):
        if isinstance(o, set):
            return tuple(o)
        elif isinstance(o, str):
            return o.encode("unicode_escape").decode(
                "ascii"
            )
        return super().default(o)


class StructuredMessage:
    def __init__(self, message, /, **kwargs):
        self.message = message
        self.kwargs = kwargs

    def __str__(self):
        s = Encoder().encode(self.kwargs)
        return "%s >>> %s" % (self.message, s)


logging.basicConfig(
    level=logging.INFO, format="%(message)s"
)
logging.info(
    StructuredMessage(
        "message 1",
        set_value={1, 2, 3},
        snowman="\u2603",
    )
)
